__author__ = 'Dan'
import pygame
import os

class AnimatedSprite(pygame.sprite.Sprite):
    def __init__(self, images, fps = 20):
        pygame.sprite.Sprite.__init__(self)
        self._images = images
        self._image_cycle_direction = 1
        self._image_facing_right = 1

        # Track the time we started, and the time between updates.
        # Then we can figure out when we have to switch the image.
        self._start = pygame.time.get_ticks()
        self._delay = 1000 / fps
        self._last_update = 0
        self._frame = 0

        # Call update to set our first image.
        self.update(pygame.time.get_ticks())

    def update(self, t, row = 0, facing = None):
        # Note that this doesn't work if it's been more that self._delay
        # time between calls to update(); we only update the image once
        # then, but it really should be updated twice.

        if t - self._last_update > self._delay:
            self._frame += self._image_cycle_direction
            if self._frame >= len(self._images[row]):
                self._image_cycle_direction = -1
                self._frame -= 1
            elif self._frame <= 0:
                self._image_cycle_direction = 1
            if facing != None:
                self._image_facing_right = facing
            if self._image_facing_right == 1:
                self.image = self._images[row][self._frame]
            else:
                self.image = pygame.transform.flip(self._images[row][self._frame], 1, 0)
            self._last_update = t

    @classmethod
    def load_sliced_sprites(cls, w, h, path):
        '''
        Specs :
            Master can be any height.
            Sprites frames width must be the same width
            Master width must be len(frames)*frame.width
        Assuming you resources directory is named "resources"
        '''
        
        master_image = pygame.image.load(path).convert_alpha()
        master_width, master_height = master_image.get_size()
        
        images = [[None for _ in xrange(master_width/w)] for __ in xrange(master_height/h)]
        for j in xrange(int(master_height/h)):
            for i in xrange(int(master_width/w)):
                images[j][i] = master_image.subsurface((i*w,j*h,w,h))
        return images