__author__ = 'Dan'

from pygame import sprite, image, transform
import pygame.time
import time
from recordtype import recordtype
from AnimatedSprite import AnimatedSprite
from Map import Tile
import sys
from audioSystem import audioSystem
from Powerup import Powerup
from threading import Timer
from resource_path import resource_path
import os

Vector = recordtype('Vector', 'x y')
Movement = recordtype('Movement', 'up down left right')

class BaseObject(sprite.Sprite):
    def __init__(self, imagePath):
        sprite.Sprite.__init__(self)
        self.animated = AnimatedSprite(AnimatedSprite.load_sliced_sprites(60, 60, resource_path(os.path.join("resources", "MonoPlayerSheet.png"))))
        self.image = self.animated.image
        self.rect = self.image.get_rect()
        self.vel = Vector(0, 0)
        self.onGround = False
        self.powerups = dict()
        for pu in Powerup.types:
            self.powerups[pu] = False
        self.powerupsConsumed = 0
        self.exitThreshold = 42
        self.xSpeed = 4
        self.xSpeedMul = 1
        self.ySpeed = 8
        self.ySpeedMul = 1
        self.health = 3
        self.invuln = False

    def setPosition(self, x, y):
        self.rect.left = x
        self.rect.top = y

    def toggleSpeedPU(self):
        self.powerups["Speed"] = not self.powerups["Speed"]
        self.powerupsConsumed -= 10 # deactivate temporary colour boost

    def toggleJumpPU(self):
        self.powerups["Jump"] = not self.powerups["Jump"]
        self.powerupsConsumed -= 10

    def toggleInvuln(self):
        self.invuln = not self.invuln

    def update(self, movement=None, collidables=None, totalLevelWidth=None):
        if self.health <= 0:
            sys.exit()
        if self.powerups["Speed"]:
            self.xSpeedMul = 2
        else:
            self.xSpeedMul = 1
        if self.powerups["Jump"]:
            self.ySpeedMul = 2
        else:
            self.ySpeedMul = 1
        if movement:
            if movement.up:
                if self.onGround:
                    self.vel.y -= self.ySpeed * self.ySpeedMul
                    song = audioSystem()
                    song.jumpSound()
                else:
                    self.vel.y -= 0.15
                self.animated.update(pygame.time.get_ticks(), 1)
                self.image = self.animated.image
            if self.rect.bottom > 600:
                sys.exit()
            if movement.down:
                pass
            if movement.left:
                self.vel.x = -self.xSpeed * self.xSpeedMul
                if not (movement.up or movement.down):
                    self.animated.update(pygame.time.get_ticks(), facing = 0)
                    self.image = self.animated.image
            if movement.right:
                self.vel.x = self.xSpeed * self.xSpeedMul
                if not (movement.up or movement.down):
                    self.animated.update(pygame.time.get_ticks(), facing = 1)
                    self.image = self.animated.image
            if not self.onGround:
                self.vel.y += 0.3
                if self.vel.y > 100:
                    self.vel.y = 100
                self.animated.update(pygame.time.get_ticks(), 1)
                self.image = self.animated.image
            if not (movement.left or movement.right):
                self.vel.x = 0
                if not (movement.up or movement.down):
                    if self.animated._image_facing_right == 1:
                        self.image = self.animated._images[0][0]
                    else:
                        self.image = transform.flip(self.animated._images[0][0], 1, 0)

            self.rect.left += self.vel.x
            if self.rect.left < 0:
                self.rect.left = 0
                self.vel.x = 0
            if self.rect.right > totalLevelWidth:
                self.rect.right = totalLevelWidth
                self.vel.x = 0
            if collidables:
                self.collide(self.vel.x, 0, collidables)
            self.rect.top += self.vel.y
            self.onGround = False
            if collidables:
                self.collide(0, self.vel.y, collidables)

    def collide(self, xvel, yvel, objects):
        for obj in objects:
            if sprite.collide_rect(self, obj):
                if xvel > 0:
                    self.rect.right = obj.rect.left
                if xvel < 0:
                    self.rect.left = obj.rect.right
                if yvel > 0:
                    self.rect.bottom = obj.rect.top
                    self.onGround = True
                    self.vel.y = 0
                if yvel < 0:
                    self.rect.top = obj.rect.bottom
                    self.vel.y = 0
                if isinstance(obj, Tile):    # check if obj is of tile class
                    if obj.tileType == "1" and self.powerupsConsumed >= self.exitThreshold:   # check if tile is an "end of level" tile
                        sys.exit()
                    if obj.damaging and not self.invuln:
                        self.toggleInvuln()
                        self.health -= 1
                        Timer(3, self.toggleInvuln, ()).start()

    
    def colourise(self, colour):
        facing = self.animated._image_facing_right
        if colour == 1:
            self.animated = AnimatedSprite(AnimatedSprite.load_sliced_sprites(60, 60, resource_path(os.path.join("resources", "RedPlayerSheet.png"))))
        elif colour == 2:
            self.animated = AnimatedSprite(AnimatedSprite.load_sliced_sprites(60, 60, resource_path(os.path.join("resources", "BluePlayerSheet.png"))))
        elif colour == 3:
            self.animated = AnimatedSprite(AnimatedSprite.load_sliced_sprites(60, 60, resource_path(os.path.join("resources", "SatPlayerSheet.png"))))
        else:
            self.animated = AnimatedSprite(AnimatedSprite.load_sliced_sprites(60, 60, resource_path(os.path.join("resources", "MonoPlayerSheet.png"))))
        self.animated._image_facing_right = facing
