__author__ = 'Jason and Chris'

from pygame import sprite
from pygame import image
from pygame import Rect
from AnimatedSprite import AnimatedSprite
from resource_path import resource_path
import os

class Map(object):
    def __init__(self, filename="Test.txt", tileset="MonoTileset.png"):
        object.__init__(self)
        mapFile = open(resource_path(os.path.join("resources", filename)), 'r')
        self.mapChars = [line.strip().split(',') for line in mapFile.readlines()]
        self.tilesetnames = ["MonoTileset", "LowSatTileset", "MidSatTileset", "HighSatTileset", "MaxSatTileset"]
        self.tilesets = dict()
        self.currentTextureSet = "MonoTileset"

        for name in self.tilesetnames:
            self.tilesets[name] = dict()
            tileset = AnimatedSprite.load_sliced_sprites(60, 60, resource_path(os.path.join("resources", "{0}.png".format(name))))
            for index, tile in enumerate(tileset[0]):
                self.tilesets[name][str(index)] = tile


        self.tileWidth = 60
        self.tileHeight = 60
        self.tileCollidable = dict()
        self.tileDamage = dict()

        self.tileCollidable["0"] = True
        self.tileCollidable["1"] = True
        self.tileCollidable["2"] = True
        self.tileCollidable["3"] = True
        self.tileCollidable["4"] = True
        self.tileCollidable["5"] = True
        self.tileCollidable["6"] = True
        self.tileCollidable["7"] = True
        self.tileCollidable["8"] = True
        self.tileCollidable["9"] = False

        self.tileDamage["0"] = False
        self.tileDamage["1"] = False
        self.tileDamage["2"] = False
        self.tileDamage["3"] = False
        self.tileDamage["4"] = False
        self.tileDamage["5"] = False
        self.tileDamage["6"] = True
        self.tileDamage["7"] = False
        self.tileDamage["8"] = True
        self.tileDamage["9"] = False

        self.tiles = []
        for rowCount, row in enumerate(self.mapChars):
            self.tiles.append([])
            for columnCount, mapChar in enumerate(row):
                self.tiles[rowCount].append(Tile(columnCount, rowCount, mapChar, self.tileDamage[mapChar]))

    def draw(self, screen, camera):
        for rowCount, row in enumerate(self.tiles):
            for columnCount, tile in enumerate(row):
                screen.blit(source=self.tilesets[self.currentTextureSet][tile.tileType], dest=camera.apply(tile))

class Tile(sprite.Sprite):
    def __init__(self, x, y, tileType, damaging):
        sprite.Sprite.__init__(self)
        self.x = x
        self.y = y
        self.tileType = tileType
        self.rect = Rect(self.x*60, self.y*60, 60, 60)
        self.damaging = damaging