#!/usr/bin/env python2
import pygame, sys
from BaseObject import BaseObject, Movement
from Map import Map
from Camera import Camera, complex_camera, simple_camera
from AnimatedSprite import AnimatedSprite
from audioSystem import audioSystem
from Powerup import Powerup
from threading import Timer
import os
from resource_path  import resource_path

__author__ = 'Dan'

SCREEN_WIDTH = 600
SCREEN_HEIGHT = 600
POWERUP_TIMEOUT = 5

def Main():
    def Draw(screen):
        screen.blit(bg, (0, 0))
        map.draw(screen, camera)
        for obj in objectList:
            screen.blit(obj.image, camera.apply(obj))
        screen.blit(player.image, camera.apply(player))
        if player.health >= 1:
            screen.blit(heart, (30, 30))
        if player.health >= 2:
            screen.blit(heart, (60, 30))
        if player.health >= 3:
            screen.blit(heart, (90, 30))
        pygame.display.flip()
    def spawnPowerup(type, gridLocation):
        pu = Powerup(type, (gridLocation[0]*60, gridLocation[1]*60))
        objectList.add(pu)

    pygame.init()
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT), 0, 32)
    clock = pygame.time.Clock()
    themeMusic = audioSystem()
    themeMusic.mainTheme()
    objectList = pygame.sprite.Group()
    collidables = pygame.sprite.Group()
    hazards = pygame.sprite.Group()
    collisions = pygame.sprite.Group()

    map = Map(filename="Level2.txt")

    bg = monobg = pygame.image.load(resource_path(os.path.join("resources", "MonochromeBG.png"))).convert()
    lowbg = pygame.image.load(resource_path(os.path.join("resources", "LowSatBG.png"))).convert()
    midbg = pygame.image.load(resource_path(os.path.join("resources", "MidSatBG.png"))).convert()
    highbg = pygame.image.load(resource_path(os.path.join("resources", "HighSatBG.png"))).convert()
    maxbg = pygame.image.load(resource_path(os.path.join("resources", "MaxSatBG.png"))).convert()
    heart = pygame.image.load(resource_path(os.path.join("resources", "Heart30x30.png"))).convert()

    for row in map.tiles:
        for tile in row:
            if map.tileCollidable[tile.tileType]:
                collidables.add(tile)
            if map.tileDamage[tile.tileType]:
                hazards.add(tile)


    playerAnimated = AnimatedSprite(AnimatedSprite.load_sliced_sprites(60, 60, resource_path(os.path.join("resources","MonoPlayerSheet.png"))))
    player = BaseObject(playerAnimated._images[0])
    player.rect.inflate_ip(-4, -4)
    player.setPosition(100,100)

    #objectList.add(player)
    up = left = right = down = False

    spawnPowerup("Speed", (64,5))
    spawnPowerup("Speed", (93,6))
    spawnPowerup("Jump", (136,7))
    spawnPowerup("Red", (5,7))
    spawnPowerup("Red", (7,6))
    spawnPowerup("Blue", (9,4))
    spawnPowerup("Red", (12,6))
    spawnPowerup("Blue", (14,7))
    spawnPowerup("Blue", (18,5))
    spawnPowerup("Blue", (22,4))
    spawnPowerup("Blue", (26,5))
    spawnPowerup("Red", (30,7))
    spawnPowerup("Red", (34,7))
    spawnPowerup("Red", (38,7))
    spawnPowerup("Blue", (42,5))
    spawnPowerup("Blue", (46,4))
    spawnPowerup("Red", (55,5))
    spawnPowerup("Blue", (58,7))
    spawnPowerup("Blue", (61,7))
    spawnPowerup("Blue", (62,5))
    spawnPowerup("Blue", (71,3))
    
    spawnPowerup("Blue", (94,7))
    spawnPowerup("Blue", (94,5))
    spawnPowerup("Blue", (95,7))
    spawnPowerup("Blue", (95,5))
    spawnPowerup("Red", (80,2))
    spawnPowerup("Red", (90,2))
    spawnPowerup("Red", (104,5))
    spawnPowerup("Blue", (110,7))
    spawnPowerup("Red", (115,3))
    spawnPowerup("Red", (115,1)) # start of "arrow" of dots
    spawnPowerup("Red", (116,1))
    spawnPowerup("Red", (117,1))
    spawnPowerup("Red", (118,0))
    spawnPowerup("Red", (118,1))
    spawnPowerup("Red", (118,2))
    spawnPowerup("Red", (119,1)) # end of arrow
    spawnPowerup("Blue", (116,7)) # start of water trap
    spawnPowerup("Blue", (118,7))
    spawnPowerup("Blue", (120,7))
    spawnPowerup("Blue", (122,7))
    spawnPowerup("Blue", (124,7)) # end of water trap
    spawnPowerup("Red", (126,6))
    spawnPowerup("Red", (128,4))
    spawnPowerup("Red", (130,2))
    
    spawnPowerup("Red", (114,4))
    spawnPowerup("Red", (115,3))
    spawnPowerup("Red", (129,7))
    spawnPowerup("Red", (130,7))
    spawnPowerup("Red", (131,7))
    
    totalLevelHeight = len(map.tiles) * 60
    totalLevelWidth = len(map.tiles[0]) * 60

    camera = Camera(complex_camera, totalLevelWidth, totalLevelHeight, SCREEN_WIDTH, SCREEN_HEIGHT)

    # Main Game Loop
    while True:
        time_passed = clock.tick(60) # default speed is 60fps
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
            if event.type == pygame.KEYDOWN:
                """Keydown."""
                if event.key == pygame.K_ESCAPE:
                    sys.exit()
                if event.key == pygame.K_RIGHT:
                    """move right"""
                    right = True
                elif event.key == pygame.K_LEFT:
                    """move left"""
                    left = True
                elif event.key == pygame.K_UP:
                    """jump"""
                    up = True
                elif event.key == pygame.K_DOWN:
                    """duck?"""
                    down = True
                # DEBUG functions: player spritesheet switching on the fly
                elif event.key == pygame.K_1:
                    player.colourise(1)
                elif event.key == pygame.K_2:
                    player.colourise(2)
                elif event.key == pygame.K_3:
                    player.colourise(3)
                elif event.key == pygame.K_0:
                    player.colourise(0)
            elif event.type == pygame.KEYUP:
                """Key Up"""
                if event.key == pygame.K_RIGHT:
                    """stop moving right"""
                    right = False
                elif event.key == pygame.K_LEFT:
                    """stop moving left"""
                    left = False
                elif event.key == pygame.K_UP:
                    """stop jumping?"""
                    up = False
                elif event.key == pygame.K_DOWN:
                    """stop ducking?"""
                    down = False
        camera.update(player)
        player.update(Movement(up=up, left=left, right=right, down=down), collidables.sprites(), totalLevelWidth)
        for obj in objectList:
            obj.update(pygame.time.get_ticks())
        for obj in objectList:
            if pygame.sprite.collide_rect(player, obj):
                if type(obj) == Powerup:
                    player.powerups[obj.type] = True
                    gettingPowerupSound = audioSystem()
                    gettingPowerupSound.collectItem()
                    player.powerupsConsumed += 1 # all collectibles add colour
                    if obj.type == "Speed":
                        player.colourise(2)
                        player.powerupsConsumed += 10 # temporary guaranteed colour boost
                        Timer(POWERUP_TIMEOUT, player.toggleSpeedPU, ()).start()
                        Timer(POWERUP_TIMEOUT, player.colourise, [0]).start()
                    if obj.type == "Jump":
                        player.colourise(1)
                        player.powerupsConsumed += 10
                        Timer(POWERUP_TIMEOUT, player.toggleJumpPU, ()).start()
                        Timer(POWERUP_TIMEOUT, player.colourise, [0]).start()
                    if player.powerupsConsumed >= 42:
                        player.colourise(3)
                    obj.kill()
 
        if 0 <= player.powerupsConsumed < 10:
            map.currentTextureSet = "MonoTileset"
            bg = monobg
        elif 11 <= player.powerupsConsumed < 21:
            map.currentTextureSet = "LowSatTileset"
            bg = lowbg
        elif 21 <= player.powerupsConsumed < 32:
            map.currentTextureSet = "MidSatTileset"
            bg = midbg
        elif 32 <= player.powerupsConsumed < 42:
            map.currentTextureSet = "HighSatTileset"
            bg = highbg
        elif 42 <= player.powerupsConsumed:
            map.currentTextureSet = "MaxSatTileset"
            bg = maxbg
        Draw(screen)
        # DEBUG: GRID LOCATION
        #print "x:", player.rect.x/60, "y:", player.rect.y/60

if __name__ == "__main__":
    Main()